import logging


class ConfigError(Exception):
    def __init__(self, message, errors=None):
        super(ConfigError, self).__init__(message)

        self.errors = errors
        logging.error(message)


class SearchException(Exception):
    def __init__(self, message, errors=None):
        super(SearchException, self).__init__(message)
        logging.error(message)


class SearchEmptyException(Exception):
    def __init__(self, message):
        super(SearchEmptyException, self).__init__(message)
        logging.error(message)


class AmqpParseException(Exception):
    def __init__(self, message, errors=None):
        super(AmqpParseException, self).__init__(message)


class CrawlException(Exception):
    def __init__(self, message, errors=None):
        super(CrawlException, self).__init__(message)


import os
import inspect
import ConfigParser

from exceptions import ConfigError


class Config():
    def __init__(self):
        self._items = {}
        self._last_accessed = None
        self.path_config = 'config'

        self.reload()

    def __getitem__(self, item):
        if not item in self._items:
            raise Exception('You are missing the \'%s\' block in your `config` file.' % item)
        return self._items[item]

    def reload(self):
        if not os.path.isfile(self.path_config):
            raise ConfigError('Could not load config file \'%s\'' % self.path_config)

        cfg = ConfigParser.ConfigParser()
        cfg.read(self.path_config)

        data = {}
        for section in cfg.sections():
            data[section] = {}

            for k, v in cfg.items(section):
                if k.startswith('#') or v.startswith('#'):
                    continue

                if not v:
                    data[section][k] = v
                    continue

                try:
                    data[section][k] = int(v) if not '.' in v else float(v)
                    continue
                except:
                    pass

                if v.lower() == 'false':
                    data[section][k] = False
                    continue
                elif v.lower() == 'true':
                    data[section][k] = True
                    continue

                data[section][k] = v

        self._items = data

    def get(self, section, item):
        try:
            return self._items[section][item]
        except:
            frm = inspect.stack()[1]
            mod = inspect.getmodule(frm[0])
            raise Exception('Could not access config variable \'%s\' from section \'%s\' - Might want to check it out - Caller: %s' % (item, section, mod.__name__))



class FileTypes():
    def __init__(self):
        self.format_movies = ['aaf', '3gp', 'asf', 'avchd', 'avi', 'cam', 'dat', 'dsh', 'flv', 'm1v', 'm2v', 'fla', 'flr', 'sol', 'm4v', 'matroska', 'wrap', 'mng', 'quicktime', 'mpeg', 'mp4', 'mxf', 'roq', 'nsv', 'ogg', 'rm', 'svi', 'smi', 'swf', 'wmv', 'mkv', 'mov']
        self.format_documents = ['602', 'abw', 'acl', 'afp', 'ami', 'amigaguide', 'ans', 'asc', 'aww', 'ccf', 'csv', 'cwk', 'dbk', 'doc', 'docm', 'docx', 'dot', 'dotx', 'egt', 'epub', 'ezw', 'fdx', 'ftm', 'ftx', 'gdoc', 'html', 'hwp', 'hwpml', 'lwp', 'mbp', 'md', 'mcw', 'mobi', 'nb', 'nbp', 'odm', 'odt', 'ott', 'omm', 'pages', 'pap', 'pdax', 'pdf', 'rtf', 'quox', 'rpt', 'sdw', 'se', 'stw', 'sxw', 'tex', 'info', 'troff', 'txt', 'uof', 'uoml', 'via', 'wpd', 'wps', 'wpt', 'wrd', 'wrf', 'wri', 'xhtml', 'xml', 'xps', '123', 'ab2', 'ab3', 'aws', 'clf', 'cell', 'csv', 'gsheet', 'numbers', 'gnumeric', 'ods', 'ots', 'qpw', 'sdc', 'slk', 'stc', 'sxc', 'tab', 'txt', 'vc', 'wk1', 'wk3', 'wk4', 'wks', 'wks', 'wq1', 'xlk', 'xls', 'xlsb', 'xlsm', 'xlsx', 'xlr', 'xlt', 'xltm', 'xlw', 'nfo', 'db']
        self.format_pictures = ['ase', 'art', 'bmp', 'blp', 'cd5', 'cit', 'cpt', 'cr2', 'cut', 'dds', 'dib', 'djvu', 'egt', 'exif', 'gif', 'gpl', 'grf', 'icns', 'ico', 'iff', 'jng', 'jpeg', 'jpg', 'jp2', 'jps', 'lbm', 'max', 'miff', 'mng', 'msp', 'nitf', 'ota', 'pbm', 'pc1', 'pc2', 'pc3', 'pcf', 'pcx', 'pdn', 'pgm', 'pi1', 'pi2', 'pi3', 'pict', 'png', 'pnm', 'pns', 'ppm', 'psb', 'psd', 'psp', 'px', 'pxm', 'pxr', 'qfx', 'raw', 'rle', 'sct', 'sgi', 'tga', 'tiff', 'tif', 'vtf', 'xbm', 'xcf', 'xpm', 'ai', 'odg', 'svg']
        self.format_music = ['8svx', '16svx', 'aiff', 'au', 'bwf', 'cdda', 'raw', 'wav', 'flac', 'la', 'pac', 'm4a', 'ape', 'optimfrog', 'rka', 'shn', 'tta', 'wv', 'wma', 'brstm', 'ast', 'amr', 'mp2', 'mp3', 'spx', 'gsm', 'wma', 'aac', 'mpc', 'vqf', 'realaudio', 'ots', 'swa', 'vox', 'voc', 'dwd', 'smp', 'aup', 'band', 'cust', 'mid', 'mus', 'sib', 'sid', 'ly', 'gym', 'vgm', 'psf', 'nsf', 'mod', 'ptb', 's3m', 'xm', 'it', 'mt2', 'mng', 'psf', 'minipsf', '', '2sf', '', 'rmj', 'spc', 'niff', 'musicxml', 'txm', 'ym', 'jam', 'asf', 'mp1', 'mscz', 'mscz', '', 'asx', 'm3u', 'pls', 'ram', 'txt/no', 'xpl', 'xspf', 'zpl', 'als', 'aup', 'band', 'cel', 'cpr', 'cwp', 'drm', 'mmr', 'npr', 'omfi', 'ses', 'sfl', 'sng', 'stf', 'snd', 'syn', 'flp']

    def get_file_format(self, file_ext):
        if not file_ext:
            return 0

        if file_ext in self.format_documents:
            return 1
        elif file_ext in self.format_movies:
            return 2
        elif file_ext in self.format_music:
            return 3
        elif file_ext in self.format_pictures:
            return 4
        else:
            return 0


#!/usr/bin/env python

"""
version 0.1 2-6-2014
url to py obj converter

Author: Sander Ferdinand' <sanderf [AT] cedsys [DOT] nl>
License: MIT
"""
import cgi
import urllib2


class ParseUrl():
    def __init__(self, uri):
        self.scheme = ''
        self.subdomain = ''
        self.fulluri = self.get_fulluri(uri)
        self.base = self.get_base(self.fulluri)
        self.domain = self.get_domain(self.base)
        self.path = self.get_path(self.fulluri)
        self.reluri = self.get_reluri(self.fulluri)

        if len(self.base.split('.')) == 2 and not self.fulluri.startswith('ftp://'):
            self.base = 'www.' + self.base
            self.fulluri = '%swww.%s' % (self.scheme, self.fulluri[len(self.scheme):])
            self.subdomain = 'www'

        if len(self.base.split('.')) == 3 and not self.fulluri.startswith('ftp://'):
            if not self.base.startswith('www.'):
                self.subdomain = self.base.split('.')[0]

        if not self.subdomain:
            self.subdomain = 'www'
        self.unquoted_uri = self.unquote_uri()

    def get_reluri(self, uri):
        uri = self.strip_scheme(uri)
        if uri.endswith('/'):
            uri = uri[:-1]

        if not '/' in uri:
            return self.scheme + uri + '/'
        if '/' in uri and not uri.endswith('/'):
            spl = uri.split('/')
            for a in ['.html', 'htm', '.php', '.aspx', '.asp']:
                if a in spl[len(spl)-1]:
                    return self.scheme + '/'.join(spl[:-1]) + '/'  # fml
            return self.scheme + '/'.join(spl) + '/'
        return '?!'

    def get_fulluri(self, uri):
        if uri.endswith('/'):
            uri = uri[:-1]

        for s in ['http://', 'https://', 'ftp://']:
            if uri.startswith(s):
                self.scheme = s
                return uri

        self.scheme = 'http://'
        return 'http://%s' % uri

    def get_base(self, uri):
        uri = self.strip_scheme(uri)
        return uri[:uri.find('/')] if '/' in uri else uri

    def get_domain(self, uri):
        spl = uri.split('.')
        if len(spl) == 4:
            for s in spl:
                try:
                    int(s)
                    return uri
                except:
                    break
        return '.'.join(spl[-2:]) if len(spl) > 1 else uri

    def get_path(self, uri):
        uri = self.strip_scheme(uri)
        return uri[uri.find('/'):] if '/' in uri else None

    def strip_scheme(self, uri):
        if uri.startswith('http://'):
            return uri[7:]
        elif uri.startswith('https://'):
            return uri[8:]
        elif uri.startswith('ftp://'):
            return uri[6:]
        else:
            return uri

    def unquote_uri(self):
        return cgi.escape(urllib2.unquote(self.fulluri))


from setuptools import setup

setup(name='findex_common',
      version='0.1.15',
      description='This package contains some modules used by different Findex projects.',
      url='http://github.com/skftn/findex-gui',
      author='Sander Ferdinand',
      author_email='sanderf@cedsys.nl',
      license='MIT',
      packages=['findex_common'],
      zip_safe=False)



from bottle import request
import cgi,urllib
import bytes2human as bytes2human
from datetime import datetime


class DataObjectManipulation():
    def __init__(self, dataobject):
        self.dataobject = dataobject

    def dictionize(self, dataobject):
        d = {}
        for attr in [a for a in dir(dataobject) if not a.startswith('__') and not a.startswith('_')]:
            d[attr] = getattr(dataobject, attr)
        return d

    def sanitize(self, dataobject):
        for attr in [a for a in dir(dataobject) if not a.startswith('__') and not a.startswith('_')]:
            get_attr = getattr(dataobject, attr)

            if isinstance(get_attr, str) or isinstance(get_attr, unicode):
                if get_attr == 'None' or get_attr == '':
                    setattr(dataobject, attr, None)
        return dataobject

    def sanitize_userinput(self, dataobject):
        if not isinstance(dataobject, dict):
            for attr in [a for a in dir(dataobject) if not a.startswith('__') and not a.startswith('_')]:
                a = getattr(dataobject, attr)
        else:
            data = {}

            def san(inp):
                    cgi.escape(urllib.quote_plus(inp))

            for k, v in dataobject.iteritems():
                if isinstance(v, list):
                    for i in range(0, len(v)):
                        dataobject[k] = san(v[i])

        return dataobject

    def humanize(self, humansizes=False, humandates=False, dateformat=None, humanpath=False, humanfile=False):
        for attr in [a for a in dir(self.dataobject) if not a.startswith('_')]:
            if humandates:
                get_attr = getattr(self.dataobject, attr)
                format = '%d %b %Y %H:%M' if not dateformat else dateformat

                if isinstance(get_attr, datetime):
                    setattr(self.dataobject, '%s_human' % attr, get_attr.strftime(format))

            if humansizes:
                get_attr = getattr(self.dataobject, attr)
                isnum = False

                if isinstance(get_attr, int):
                    isnum = True
                elif isinstance(get_attr, long):
                    isnum = True

                if isnum:
                    tokens = ['filesize', 'file_size', 'bytes', 'total_size', 'size_files']
                    if attr in tokens:
                        setattr(self.dataobject, attr + '_human', bytes2human.bytes2human(get_attr))

            if humanpath or humanfile:
                get_attr = getattr(self.dataobject, attr)

                if isinstance(get_attr, str) or isinstance(get_attr, unicode):
                    tokens = ['file_path', 'file_name']
                    if attr in tokens:
                        setattr(self.dataobject, attr + '_human', urllib.unquote_plus(get_attr.encode('utf-8')))
        return self.dataobject


class Types():
    def is_int(self, x):
        try:
            if isinstance(x, int):
                return x
            return int(x)
        except Exception as e:
            return

    def is_float(self, x):
        try:
            if isinstance(x, float):
                return x
            return float(x)
        except Exception as e:
            return


class ArgValidate():
    def __init__(self):
        self._types = Types()
        self._uri = 'post'

    def post_get(self, name, default=''):
        return request.POST.get(name, default)

    def get_get(self, name):
        if name in request.params.dict:
            if request.params.dict[name]:
                if request.params.dict[name][0]:
                    return request.params.dict[name][0]

    def verify_args(self, args, method):
        data = {}

        for k, v in args.iteritems():

            if method.lower() == 'get':
                result = self.get_get(k)
            else:
                result = self.post_get(k)

            if result:
                if v == 'float':
                    if self._types.is_float(result):
                        data[k] = float(result)
                        continue
                    else:
                        return '%s was not of type %s' % (result, v)
                elif v == 'int':
                    result = self._types.is_int(result)

                    if result:
                        data[k] = int(result)
                    elif result == 0 and isinstance(result, int):
                        data[k] = int(result)
                    else:
                        return '%s was not of type %s' % (result, v)
                else:
                    data[k] = result
            else:
                return '%s not found as arg' % k

        return data



#!/usr/bin/env python

"""
Bytes-to-human / human-to-bytes converter.
Based on: http://goo.gl/kTQMs
Working with Python 2.x and 3.x.

Author: Giampaolo Rodola' <g.rodola [AT] gmail [DOT] com>
License: MIT
"""

# see: http://goo.gl/kTQMs
SYMBOLS = {
    'customary'     : ('B', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'),
    'customary_ext' : ('byte', 'kilo', 'mega', 'giga', 'tera', 'peta', 'exa',
                       'zetta', 'iotta'),
    'iec'           : ('Bi', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi'),
    'iec_ext'       : ('byte', 'kibi', 'mebi', 'gibi', 'tebi', 'pebi', 'exbi',
                       'zebi', 'yobi'),
    }

def bytes2human(n, format='%(value).1f %(symbol)s', symbols='customary'):
    """
    Convert n bytes into a human readable string based on format.
    symbols can be either "customary", "customary_ext", "iec" or "iec_ext",
    see: http://goo.gl/kTQMs

      >>> bytes2human(0)
      '0.0 B'
      >>> bytes2human(0.9)
      '0.0 B'
      >>> bytes2human(1)
      '1.0 B'
      >>> bytes2human(1.9)
      '1.0 B'
      >>> bytes2human(1024)
      '1.0 K'
      >>> bytes2human(1048576)
      '1.0 M'
      >>> bytes2human(1099511627776127398123789121)
      '909.5 Y'

      >>> bytes2human(9856, symbols="customary")
      '9.6 K'
      >>> bytes2human(9856, symbols="customary_ext")
      '9.6 kilo'
      >>> bytes2human(9856, symbols="iec")
      '9.6 Ki'
      >>> bytes2human(9856, symbols="iec_ext")
      '9.6 kibi'

      >>> bytes2human(10000, "%(value).1f %(symbol)s/sec")
      '9.8 K/sec'

      >>> # precision can be adjusted by playing with %f operator
      >>> bytes2human(10000, format="%(value).5f %(symbol)s")
      '9.76562 K'
    """
    n = int(n)
    if n < 0:
        raise ValueError("n < 0")
    symbols = SYMBOLS[symbols]
    prefix = {}
    for i, s in enumerate(symbols[1:]):
        prefix[s] = 1 << (i+1)*10
    for symbol in reversed(symbols[1:]):
        if n >= prefix[symbol]:
            value = float(n) / prefix[symbol]
            return format % locals()
    return format % dict(symbol=symbols[0], value=n)

def human2bytes(s):
    """
    Attempts to guess the string format based on default symbols
    set and return the corresponding bytes as an integer.
    When unable to recognize the format ValueError is raised.

      >>> human2bytes('0 B')
      0
      >>> human2bytes('1 K')
      1024
      >>> human2bytes('1 M')
      1048576
      >>> human2bytes('1 Gi')
      1073741824
      >>> human2bytes('1 tera')
      1099511627776

      >>> human2bytes('0.5kilo')
      512
      >>> human2bytes('0.1  byte')
      0
      >>> human2bytes('1 k')  # k is an alias for K
      1024
      >>> human2bytes('12 foo')
      Traceback (most recent call last):
          ...
      ValueError: can't interpret '12 foo'
    """
    init = s
    num = ""
    while s and s[0:1].isdigit() or s[0:1] == '.':
        num += s[0]
        s = s[1:]
    num = float(num)
    letter = s.strip()
    for name, sset in SYMBOLS.items():
        if letter in sset:
            break
    else:
        if letter == 'k':
            # treat 'k' as an alias for 'K' as per: http://goo.gl/kTQMs
            sset = SYMBOLS['customary']
            letter = letter.upper()
        else:
            raise ValueError("can't interpret %r" % init)
    prefix = {sset[0]:1}
    for i, s in enumerate(sset[1:]):
        prefix[s] = 1 << (i+1)*10
    return int(num * prefix[letter])


if __name__ == "__main__":
    import doctest
    doctest.testmod()
